var {Promise, map} = require('..'),
	{parse} = require('esprima'),
	es = require('event-stream'),
	{Stream} = require('stream'),
	{traverse} = require('estraverse'),
	{SourceMapConsumer} = require('source-map'),
	{relative} = require('path');

function whenVinyl(file) {
	if (file.contents instanceof Buffer) {
		return Promise.resolve(file);
	} else
	if (file.contents instanceof Stream) {
		var whenContents = Promise.defer();
		file.contents.pipe(es.wait(whenContents.callback));
		return whenContents.then(contents => ({path: file.path, contents: new Buffer(contents)}));
	} else {
		return Promise.reject(new Error('File content could not be retrieved.'));
	}
}

module.exports = pairs => map(pairs, vinylPair => {
	var whenCode = whenVinyl(vinylPair.code),
		whenMap = vinylPair.map && whenVinyl(vinylPair.map);

	return whenCode.then(code => {
		var program = parse(code.contents, {loc: true, source: code.path});

		var ast = {
			type: 'File',
			program,
			loc: JSON.parse(JSON.stringify(program.loc))
		};

		if (!whenMap) {
			return ast;
		}

		return whenMap.then(map => {
			map = new SourceMapConsumer(JSON.parse(map.contents));
			
			traverse(program, {
				enter: node => {
					if (!node.loc) return;

					var {line, column, source, name} = map.originalPositionFor(node.loc.start);
					
					if (!line) {
						delete node.loc;
						return;
					}

					node.loc = {
						start: {line, column},
						source,
						name
					};
				}
			});

			return ast;
		});
	});
});