var Promise = (map = require('..')).Promise, map = map.map,
	parse = (require('esprima')).parse,
	es = require('event-stream'),
	Stream = (require('stream')).Stream,
	traverse = (require('estraverse')).traverse,
	SourceMapConsumer = (require('source-map')).SourceMapConsumer,
	relative = (require('path')).relative;

function whenVinyl(file) {
	if (file.contents instanceof Buffer) {
		return Promise.resolve(file);
	} else
	if (file.contents instanceof Stream) {
		var whenContents = Promise.defer();
		file.contents.pipe(es.wait(whenContents.callback));
		return whenContents.then(function(contents ) {return {path: file.path, contents: new Buffer(contents)}} );
	} else {
		return Promise.reject(new Error('File content could not be retrieved.'));
	}
}

module.exports = function(pairs ) {return map(pairs, function(vinylPair ) {
	var whenCode = whenVinyl(vinylPair.code),
		whenMap = vinylPair.map && whenVinyl(vinylPair.map);

	return whenCode.then(function(code ) {
		var program = parse(code.contents, {loc: true, source: code.path});

		var ast = {
			type: 'File',
			program: program,
			loc: JSON.parse(JSON.stringify(program.loc))
		};

		if (!whenMap) {
			return ast;
		}

		return whenMap.then(function(map ) {
			map = new SourceMapConsumer(JSON.parse(map.contents));
			
			traverse(program, {
				enter: function(node ) {
					if (!node.loc) return;

					var line = (name = map.originalPositionFor(node.loc.start)).line, column = name.column, source = name.source, name = name.name;
					
					if (!line) {
						delete node.loc;
						return;
					}

					node.loc = {
						start: {line: line, column: column},
						source: source,
						name: name
					};
				}
			});

			return ast;
		});
	});
})};